/*
 * file CcVob.java
 *
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM 
 *
 * com.ibm.rational.wvcm.stp.cc.CcVob
 *
 * (C) Copyright IBM Corporation 2004, 2012.  All Rights Reserved.
 * Note to U.S. Government Users Restricted Rights:  Use, duplication or 
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

package com.ibm.rational.wvcm.stp.cc;

import static com.ibm.rational.wvcm.stpex.StpExBase.PROPERTY_NAMESPACE;

import javax.wvcm.PropertyNameList.PropertyName;
import javax.wvcm.ResourceList;
import javax.wvcm.WvcmException;

import com.ibm.rational.wvcm.stp.StpRepository;

/**
 * <p>
 * A proxy for a ClearCase versioned object base (VOB) - the repository for all
 * server-side ClearCase resources.
 * </p>
 * <p>
 * A VOB can either be a regular VOB or project VOB. A project VOB is a
 * partcular type of VOB that may contain UCM project metadata - projects,
 * streams, activities, components, and baselines.
 * </p>
 */
public interface CcVob extends StpRepository, CcVobResource
{
    /** List of attribute types in this VOB. Includes global types. */
    PropertyName<ResourceList<CcAttributeType>> ATTRIBUTE_TYPE_LIST =
        new PropertyName<ResourceList<CcAttributeType>>(PROPERTY_NAMESPACE,
                                                        "attribute-type-list");

    /**
     * Get the value of this proxy's {@link #ATTRIBUTE_TYPE_LIST} property. Note
     * that this includes global types higher up the Admin VOB 
     * hierarchy with no local copy in this VOB.
     * 
     * @return list of attribute types in this VOB as AttributeType instances.
     * @throws WvcmException if this proxy doesn't define a value for this
     *             property.
     */
    public ResourceList<CcAttributeType> getAttributeTypeList()
        throws WvcmException;

    /** List of branch types in this VOB. Includes global types. */
    PropertyName<ResourceList<CcBranchType>> BRANCH_TYPE_LIST =
        new PropertyName<ResourceList<CcBranchType>>(PROPERTY_NAMESPACE,
                                                     "branch-type-list");

    /**
     * Get the value of this proxy's {@link #BRANCH_TYPE_LIST} property. Note
     * that this includes global types higher up the Admin VOB 
     * hierarchy with no local copy in this VOB.
     * 
     * @return list of branch types in this VOB as BranchType instances.
     * @throws WvcmException if this proxy doesn't define a value for this
     *             property.
     */
    public ResourceList<CcBranchType> getBranchTypeList() throws WvcmException;

    /**
     * List of UCM components in this project VOB.
     * This property is only supported on project VOBs.
     */
    PropertyName<ResourceList<CcComponent>> COMPONENT_LIST =
        new PropertyName<ResourceList<CcComponent>>(PROPERTY_NAMESPACE,
                                                    "component-list");

    /**
     * Get the value of this proxy's {@link #COMPONENT_LIST} property.
     * 
     * @return list of components in this VOB as UcmComponent instances.
     * @throws WvcmException if this proxy doesn't define a value for this
     *             property.
     */
    public ResourceList<CcComponent> getComponentList() throws WvcmException;

    /** List of element types in this VOB. Includes global types. */
    PropertyName<ResourceList<CcElementType>> ELEMENT_TYPE_LIST =
        new PropertyName<ResourceList<CcElementType>>(PROPERTY_NAMESPACE,
                                                      "element-type-list");

    /**
     * Get the value of this proxy's {@link #ELEMENT_TYPE_LIST} property. Note
     * that this includes global types higher up the Admin VOB 
     * hierarchy with no local copy in this VOB.
     * 
     * @return list of element types in this VOB as ElementType instances.
     * @throws WvcmException if this proxy doesn't define a value for this
     *             property.
     */
    public ResourceList<CcElementType> getElementTypeList()
        throws WvcmException;

    /** List of hyperlink types in this VOB. Includes global types. */
    PropertyName<ResourceList<CcHyperlinkType>> HYPERLINK_TYPE_LIST =
        new PropertyName<ResourceList<CcHyperlinkType>>(PROPERTY_NAMESPACE,
                                                        "hyperlink-type-list");

    /**
     * Get the value of this proxy's {@link #HYPERLINK_TYPE_LIST} property. Note
     * that this includes global types higher up the Admin VOB 
     * hierarchy with no local copy in this VOB.
     * 
     * @return list of hyperlink types in this VOB as HyperlinkType instances.
     * @throws WvcmException if this proxy doesn't define a value for this
     *             property.
     */
    public ResourceList<CcHyperlinkType> getHyperlinkTypeList()
        throws WvcmException;

    /** Does this VOB have ACLs enforced? */
    PropertyName<Boolean> IS_ACL_ENFORCED = 
            new PropertyName<Boolean>(PROPERTY_NAMESPACE,
                    "is-acl-enforced");

    /**
     * Returns the value of this proxy's {@link #IS_ACL_ENFORCED} property.
     * 
     * @return true if this VOB has ACL enforcement turned on, else false
     * @throws WvcmException if this proxy doesn't define a value for this
     *             property.
     */
    boolean getIsAclEnforced() throws WvcmException;

    /** Is this VOB a project VOB? */
    PropertyName<Boolean> IS_PROJECT_VOB =
        new PropertyName<Boolean>(PROPERTY_NAMESPACE,
                                  "is-project-vob");

    /**
     * Returns the value of this proxy's {@link #IS_PROJECT_VOB} property.
     * 
     * @return true if this VOB is a project VOB, else false
     * @throws WvcmException if this proxy doesn't define a value for this
     *             property.
     */
    boolean getIsProjectVob() throws WvcmException;

    /** Is this VOB replicated? */
    PropertyName<Boolean> IS_REPLICATED =
        new PropertyName<Boolean>(PROPERTY_NAMESPACE, "is-replicated");

    /**
     * Returns the value of this proxy's {@link #IS_REPLICATED} property.
     * 
     * @return true if the VOB is replicated, else false
     * @throws WvcmException
     *             if this proxy doesn't define a value for this property.
     */
    boolean getIsReplicated() throws WvcmException;

    /** List of label types in this VOB. Includes global types. */
    PropertyName<ResourceList<CcLabelType>> LABEL_TYPE_LIST =
        new PropertyName<ResourceList<CcLabelType>>(PROPERTY_NAMESPACE,
                                                    "label-type-list");

    /**
     * Get the value of this proxy's {@link #LABEL_TYPE_LIST} property. Note
     * that this includes global types higher up the Admin VOB 
     * hierarchy with no local copy in this VOB.
     * 
     * @return list of label types in this VOB as LabelType instances.
     * @throws WvcmException if this proxy doesn't define a value for this
     *             property.
     */
    public ResourceList<CcLabelType> getLabelTypeList() throws WvcmException;

    /** 
     * Ordered list of Admin VOBs for this VOB with the immediate parent first 
     * and the furthest ancestor last.
     */
    PropertyName<ResourceList<CcVob>> ORDERED_ADMIN_VOB_LIST =
        new PropertyName<ResourceList<CcVob>>(PROPERTY_NAMESPACE,
                                                      "ordered-admin-vob-list");

    /**
     * Get the value of this proxy's {@link #ORDERED_ADMIN_VOB_LIST} property.
     * 
     * @return list of Admin VOBs for this VOB
     * @throws WvcmException if this proxy doesn't define a value for this
     *             property.
     */
    public ResourceList<CcVob> getOrderedAdminVobList() throws WvcmException;

    /** List of policies in this VOB */
    PropertyName<ResourceList<CcPolicy>> POLICY_LIST =
        new PropertyName<ResourceList<CcPolicy>>(PROPERTY_NAMESPACE,
                                                      "policy-list");

    /**
     * Get the value of this proxy's {@link #POLICY_LIST} property.
     * 
     * @return list of policies in this VOB as CcPolicy instances.
     * @throws WvcmException if this proxy doesn't define a value for this
     *             property.
     */
    public ResourceList<CcPolicy> getPolicyList() throws WvcmException;

    /** List of rolemaps in this VOB */
    PropertyName<ResourceList<CcRolemap>> ROLEMAP_LIST =
        new PropertyName<ResourceList<CcRolemap>>(PROPERTY_NAMESPACE,
                                                      "rolemap-list");

    /**
     * Get the value of this proxy's {@link #ROLEMAP_LIST} property.
     * 
     * @return list of rolemaps in this VOB as CcRolemap instances.
     * @throws WvcmException if this proxy doesn't define a value for this
     *             property.
     */
    public ResourceList<CcRolemap> getRolemapList() throws WvcmException;

    /**
     * This project VOB's root UCM project folder.
     * This property is only supported on project VOBs.
     */
    PropertyName<CcProjectFolder> ROOT_PROJECT_FOLDER =
        new PropertyName<CcProjectFolder>(PROPERTY_NAMESPACE,
                                          "root-project-folder");

    /**
     * Returns the value of this proxy's {@link #ROOT_PROJECT_FOLDER} property.
     * 
     * @return A client proxy for this project VOB's root project folder
     * @throws WvcmException if this proxy doesn't define a value for this
     *             property.
     */
    public CcProjectFolder getRootProjectFolder() throws WvcmException;

    /**
     * This VOB's db schema version.
     */
    PropertyName<Long> SCHEMA_VERSION =
        new PropertyName<Long>(PROPERTY_NAMESPACE, "schema-version");
    
    /**
     * Get the value of this proxy's {@link #SCHEMA_VERSION} property.
     * @return long indicating VOB's schema version
     * @throws WvcmException if this proxy doesn't define a value for this
     *             property.
     */
    public long getSchemaVersion() throws WvcmException;
    
    /** List of trigger types in this VOB */
    PropertyName<ResourceList<CcTriggerType>> TRIGGER_TYPE_LIST =
        new PropertyName<ResourceList<CcTriggerType>>(PROPERTY_NAMESPACE,
                                                      "trigger-type-list");

    /**
     * Get the value of this proxy's {@link #TRIGGER_TYPE_LIST} property.
     * 
     * @return list of trigger types in this VOB as TriggerType instances.
     * @throws WvcmException if this proxy doesn't define a value for this
     *             property.
     */
    public ResourceList<CcTriggerType> getTriggerTypeList()
        throws WvcmException;

    /** This VOB's VOB tag as a string in the current registry region. */
    PropertyName<String> VOB_TAG_STRING = new PropertyName<String>(PROPERTY_NAMESPACE,
                                                            "vob-tag-string");

    /**
     * Returns the value of this proxy's {@link #VOB_TAG_STRING} property.
     * 
     * @return this VOB's VOB tag
     * @throws WvcmException if this proxy doesn't define a value for this
     *             property.
     */
    public String getVobTagString() throws WvcmException;
    
    /** 
     * This VOB's VOB tag as a {@link CcVobTag} resource in the current
     * registry region. 
     */
    PropertyName<CcVobTag> VOB_TAG = 
        new PropertyName<CcVobTag>(PROPERTY_NAMESPACE, "vob-tag");
    
    /**
     * Returns the value of this proxy's {@link #VOB_TAG} property.
     * 
     * @return this VOB's VOB tag
     * @throws WvcmException if this proxy doesn't define a value for this
     *             property.
     */
    public CcVobTag getVobTag() throws WvcmException;

    /**
     * The list of MultiSite replicas of this VOB.
     * This list may be incomplete; other replicas may exist, but their
     * creation packets have not yet been imported at the current replica.
     */
    PropertyName<ResourceList<CcReplica>> REPLICA_LIST =
        new PropertyName<ResourceList<CcReplica>>(
                PROPERTY_NAMESPACE, "replica-list");

    /**
     * Returns the value of this proxy's {@link #REPLICA_LIST} property.
     * 
     * @return this VOB's list of replicas as a list of CcReplica instances
     * @throws WvcmException if this proxy doesn't define a value for this
     *             property.
     */
    public ResourceList<CcReplica> getReplicaList() throws WvcmException;

    /**
     * Check to see if a type with the given name is visible in this VOB.
     * To be visible a type must be an ordinary or local type defined in the VOB
     * or a global type in the AdminVOB hierarchy above this VOB which has
     * not yet been instantiated.
     * @param labelName  Name of the label to look for.
     * @return  true if a label with that name is visible from this VOB, else false.
     */
    public boolean doIsLabelTypeVisible(String labelName) throws WvcmException;
    
    /** Is atomic checkin enabled in this VOB? */
    PropertyName<Boolean> IS_ATOMIC_CHECKIN_ENABLED =
        new PropertyName<Boolean>(PROPERTY_NAMESPACE, "is-atomic-checkin-enabled");
    
    /**
     * Returns the value of this proxy's {@link #IS_ATOMIC_CHECKIN_ENABLED} property.
     * 
     * @return true if atomic checkin is enabled in this VOB; false otherwise
     * @throws WvcmException if this proxy doesn't define a value for this
     *             property.
     */
    public boolean getIsAtomicCheckinEnabled() throws WvcmException;
            
}